@@ -33,13 +33,12 @@ def eprint(*strings):
 
 
 def check_file(path: Path) -> bool:
-    print(f"Checking {path.absolute()}...")  # noqa: T001
     file = path.open()
     valid = True
 
     for i, line in enumerate(file, start=1):
         if todo_re.match(line) and not accounted_for_todo.match(line):
-            eprint(f"{i}: {line}")
+            eprint(f"{path.absolute()}:{i}: {line}")
             valid = False
 
     file.close()
@@ -64,14 +63,6 @@ with open(Path("sublime_music/__init__.py")) as f:
     else:  # nobreak
         raise AssertionError("No version in sublime_music/__init__.py")
 
-with open(Path("pyproject.toml")) as f:
-    for line in f:
-        if line.startswith("version ="):
-            assert eval(line.split()[-1]) == version, "Version mismatch: pyproject.toml"
-            break
-    else:  # nobreak
-        raise AssertionError("No version in pyproject.toml")
-
 with open(Path("CHANGELOG.rst")) as f:
     assert f.readline().strip() == f"v{version}", "Version mismatch: CHANGELOG"
 
