@@ -6,29 +6,17 @@ import multiprocessing
 import os
 import socket
 from datetime import timedelta
-from typing import Any, Callable, cast, Dict, Optional, Set, Tuple, Type, Union
+from typing import Any, Callable, Dict, Optional, Set, Tuple, Type, Union, cast
 from urllib.parse import urlparse
 from uuid import UUID
 
+import bottle
+import pychromecast
 from gi.repository import GLib
 
-from .base import Player, PlayerDeviceEvent, PlayerEvent
 from ..adapters import AdapterManager
 from ..adapters.api_objects import Song
-
-try:
-    import pychromecast
-
-    chromecast_imported = True
-except Exception:
-    chromecast_imported = False
-
-try:
-    import bottle
-
-    bottle_imported = True
-except Exception:
-    bottle_imported = False
+from .base import Player, PlayerDeviceEvent, PlayerEvent
 
 SERVE_FILES_KEY = "Serve Local Files to Chromecasts on the LAN"
 LAN_PORT_KEY = "LAN Server Port Number"
@@ -40,18 +28,16 @@ class ChromecastPlayer(Player):
 
     @property
     def enabled(self) -> bool:
-        return chromecast_imported
+        return True
 
     @staticmethod
     def get_configuration_options() -> Dict[str, Union[Type, Tuple[str, ...]]]:
-        if not bottle_imported:
-            return {}
         return {SERVE_FILES_KEY: bool, LAN_PORT_KEY: int}
 
     @property
     def supported_schemes(self) -> Set[str]:
         schemes = {"http", "https"}
-        if bottle_imported and self.config.get(SERVE_FILES_KEY):
+        if self.config.get(SERVE_FILES_KEY):
             schemes.add("file")
         return schemes
 
@@ -73,31 +59,27 @@ class ChromecastPlayer(Player):
 
         self.change_settings(config)
 
-        if chromecast_imported:
-            self._chromecasts: Dict[UUID, pychromecast.Chromecast] = {}
-            self._current_chromecast: Optional[pychromecast.Chromecast] = None
+        self._chromecasts: Dict[UUID, pychromecast.Chromecast] = {}
+        self._current_chromecast: Optional[pychromecast.Chromecast] = None
 
-            self.stop_get_chromecasts = None
-            self.refresh_players()
+        self.chromecast_browser = None
+        self.refresh_players()
 
     def chromecast_discovered_callback(self, chromecast: Any):
         chromecast = cast(pychromecast.Chromecast, chromecast)
-        self._chromecasts[chromecast.device.uuid] = chromecast
+        self._chromecasts[chromecast.cast_info.uuid] = chromecast
         self.player_device_change_callback(
             PlayerDeviceEvent(
                 PlayerDeviceEvent.Delta.ADD,
                 type(self),
-                str(chromecast.device.uuid),
-                chromecast.device.friendly_name,
+                str(chromecast.cast_info.uuid),
+                chromecast.cast_info.friendly_name,
             )
         )
 
     def change_settings(self, config: Dict[str, Union[str, int, bool]]):
-        if not chromecast_imported:
-            return
-
         self.config = config
-        if bottle_imported and self.config.get(SERVE_FILES_KEY):
+        if self.config.get(SERVE_FILES_KEY):
             # Try and terminate the existing process if it exists.
             if self.server_process is not None:
                 try:
@@ -112,25 +94,18 @@ class ChromecastPlayer(Player):
             self.server_process.start()
 
     def refresh_players(self):
-        if not chromecast_imported:
-            return
-
-        if self.stop_get_chromecasts is not None:
-            self.stop_get_chromecasts.cancel()
-
         for id_, chromecast in self._chromecasts.items():
             self.player_device_change_callback(
                 PlayerDeviceEvent(
                     PlayerDeviceEvent.Delta.REMOVE,
                     type(self),
                     str(id_),
-                    chromecast.device.friendly_name,
+                    chromecast.cast_info.friendly_name,
                 )
             )
-
         self._chromecasts = {}
 
-        self.stop_get_chromecasts = pychromecast.get_chromecasts(
+        self.chromecast_browser = pychromecast.get_chromecasts(
             blocking=False, callback=self.chromecast_discovered_callback
         )
 
@@ -145,7 +120,7 @@ class ChromecastPlayer(Player):
         self.on_player_event(
             PlayerEvent(
                 PlayerEvent.EventType.VOLUME_CHANGE,
-                str(self._current_chromecast.device.uuid),
+                str(self._current_chromecast.cast_info.uuid),
                 volume=(status.volume_level * 100 if not status.volume_muted else 0),
             )
         )
@@ -177,7 +152,7 @@ class ChromecastPlayer(Player):
         self.on_player_event(
             PlayerEvent(
                 PlayerEvent.EventType.PLAY_STATE_CHANGE,
-                str(self._current_chromecast.device.uuid),
+                str(self._current_chromecast.cast_info.uuid),
                 playing=(status.player_state in ("PLAYING", "BUFFERING")),
             )
         )
@@ -193,11 +168,15 @@ class ChromecastPlayer(Player):
         self.time_increment_order_token += 1
         GLib.timeout_add(500, increment_time, self.time_increment_order_token)
 
+    def reset(self):
+        pass
+
     def shutdown(self):
         if self.server_process:
             self.server_process.terminate()
 
         try:
+            assert self._current_chromecast
             self._current_chromecast.quit_app()
         except Exception:
             pass
@@ -220,11 +199,12 @@ class ChromecastPlayer(Player):
 
         @app.route("/s/<token>")
         def stream_song(token: str) -> bytes:
-            if token != self._serving_token.value.decode():
+            # typing doesn't support multiprocessing.Value very well
+            if token != self._serving_token.value.decode():  # type: ignore
                 raise bottle.HTTPError(status=401, body="Invalid token.")
 
             song = AdapterManager.get_song_details(
-                self._serving_song_id.value.decode()
+                self._serving_song_id.value.decode()  # type: ignore
             ).result()
             filename = AdapterManager.get_song_file_uri(song)
             with open(filename[7:], "rb") as fin:
@@ -239,15 +219,12 @@ class ChromecastPlayer(Player):
 
     @property
     def playing(self) -> bool:
-        if (
-            not self._current_chromecast
-            or not self._current_chromecast.media_controller
-        ):
+        if not self._current_chromecast or not self._current_chromecast.media_controller:
             return False
         return self._current_chromecast.media_controller.status.player_is_playing
 
     def get_volume(self) -> float:
-        if self._current_chromecast:
+        if self._current_chromecast and self._current_chromecast.status:
             # The volume is in the range [0, 1]. Multiply by 100 to get to [0, 100].
             return self._current_chromecast.status.volume_level * 100
         else:
@@ -273,8 +250,9 @@ class ChromecastPlayer(Player):
         scheme = urlparse(uri).scheme
         if scheme == "file":
             token = base64.b16encode(os.urandom(8))
-            self._serving_token.value = token
-            self._serving_song_id.value = song.id.encode()
+            # typing doesn't support multiprocessing.Value very well
+            self._serving_token.value = token  # type: ignore
+            self._serving_song_id.value = song.id.encode()  # type: ignore
 
             # If this fails, then we are basically screwed, so don't care if it blows
             # up.
@@ -286,7 +264,7 @@ class ChromecastPlayer(Player):
             s.close()
 
             uri = f"http://{host_ip}:{self.config.get(LAN_PORT_KEY)}/s/{token.decode()}"
-            logging.info("Serving {song.name} at {uri}")
+            logging.info(f"Serving {song.title} at {uri}")
 
         assert AdapterManager._instance
         networked_scheme_priority = ("https", "http")
@@ -316,7 +294,7 @@ class ChromecastPlayer(Player):
         self.on_player_event(
             PlayerEvent(
                 PlayerEvent.EventType.STREAM_CACHE_PROGRESS_CHANGE,
-                str(self._current_chromecast.device.uuid),
+                str(self._current_chromecast.cast_info.uuid),
                 stream_cache_duration=0,
             )
         )
@@ -341,3 +319,6 @@ class ChromecastPlayer(Player):
 
     def _wait_for_playing(self):
         pass
+
+    def next_media_cached(self, *_):
+        pass
