@@ -1,6 +1,7 @@
 from .album_with_songs import AlbumWithSongs
 from .icon_button import IconButton, IconMenuButton, IconToggleButton
 from .load_error import LoadError
+from .rating_button import RatingButtonBox
 from .song_list_column import SongListColumn
 from .spinner_image import SpinnerImage
 
@@ -10,6 +11,7 @@ __all__ = (
     "IconMenuButton",
     "IconToggleButton",
     "LoadError",
+    "RatingButtonBox",
     "SongListColumn",
     "SpinnerImage",
 )
