@@ -5,16 +5,15 @@ from functools import partial
 from typing import Any, Callable, Dict, Optional, Set, Tuple
 
 import bleach
-
 from gi.repository import Gdk, GdkPixbuf, GLib, GObject, Gtk, Pango
 
-from . import util
-from .common import IconButton, IconToggleButton, SpinnerImage
-from .state import RepeatType
 from ..adapters import AdapterManager, Result, SongCacheStatus
 from ..adapters.api_objects import Song
 from ..config import AppConfiguration
 from ..util import resolve_path
+from . import util
+from .common import IconButton, IconToggleButton, RatingButtonBox, SpinnerImage
+from .state import RepeatType
 
 
 class PlayerControls(Gtk.ActionBar):
@@ -31,6 +30,7 @@ class PlayerControls(Gtk.ActionBar):
             GObject.TYPE_NONE,
             (int, object, object),
         ),
+        "song-rated": (GObject.SignalFlags.RUN_FIRST, GObject.TYPE_NONE, ()),
         "songs-removed": (GObject.SignalFlags.RUN_FIRST, GObject.TYPE_NONE, (object,)),
         "refresh-window": (
             GObject.SignalFlags.RUN_FIRST,
@@ -53,15 +53,17 @@ class PlayerControls(Gtk.ActionBar):
         Gtk.ActionBar.__init__(self)
         self.set_name("player-controls-bar")
 
+        if AdapterManager.can_get_song_rating():
+            self.create_rating_buttons()
         song_display = self.create_song_display()
         playback_controls = self.create_playback_controls()
-        play_queue_volume = self.create_play_queue_volume()
+        rating_play_queue_volume = self.create_rating_play_queue_volume()
 
         self.last_device_list_update = None
 
         self.pack_start(song_display)
         self.set_center_widget(playback_controls)
-        self.pack_end(play_queue_volume)
+        self.pack_end(rating_play_queue_volume)
 
     connecting_to_device_token = 0
     connecting_icon_index = 0
@@ -71,24 +73,16 @@ class PlayerControls(Gtk.ActionBar):
         self.update_device_list(app_config)
 
         duration = (
-            app_config.state.current_song.duration
-            if app_config.state.current_song
-            else None
+            app_config.state.current_song.duration if app_config.state.current_song else None
         )
         song_stream_cache_progress = (
-            app_config.state.song_stream_cache_progress
-            if app_config.state.current_song
-            else None
-        )
-        self.update_scrubber(
-            app_config.state.song_progress, duration, song_stream_cache_progress
+            app_config.state.song_stream_cache_progress if app_config.state.current_song else None
         )
+        self.update_scrubber(app_config.state.song_progress, duration, song_stream_cache_progress)
 
         icon = "pause" if app_config.state.playing else "start"
         self.play_button.set_icon(f"media-playback-{icon}-symbolic")
-        self.play_button.set_tooltip_text(
-            "Pause" if app_config.state.playing else "Play"
-        )
+        self.play_button.set_tooltip_text("Pause" if app_config.state.playing else "Play")
 
         has_current_song = app_config.state.current_song is not None
         has_next_song = False
@@ -156,9 +150,7 @@ class PlayerControls(Gtk.ActionBar):
         self.volume_mute_toggle.set_icon(f"audio-volume-{icon_name}-symbolic")
 
         self.editing = True
-        self.volume_slider.set_value(
-            0 if app_config.state.is_muted else app_config.state.volume
-        )
+        self.volume_slider.set_value(0 if app_config.state.is_muted else app_config.state.volume)
         self.editing = False
 
         # Update the current song information.
@@ -169,10 +161,10 @@ class PlayerControls(Gtk.ActionBar):
                 app_config.state.current_song.cover_art,
                 order_token=self.cover_art_update_order_token,
             )
+            if AdapterManager.can_get_song_rating():
+                self.update_rating(app_config.state.current_song.user_rating)
 
-            self.song_title.set_markup(
-                bleach.clean(app_config.state.current_song.title)
-            )
+            self.song_title.set_markup(bleach.clean(app_config.state.current_song.title))
             # TODO (#71): use walrus once MYPY gets its act together
             album = app_config.state.current_song.album
             artist = app_config.state.current_song.artist
@@ -221,9 +213,7 @@ class PlayerControls(Gtk.ActionBar):
             self.popover_label.set_markup("<b>Play Queue</b>")
         else:
             song_label = util.pluralize("song", play_queue_len)
-            self.popover_label.set_markup(
-                f"<b>Play Queue:</b> {play_queue_len} {song_label}"
-            )
+            self.popover_label.set_markup(f"<b>Play Queue:</b> {play_queue_len} {song_label}")
 
         # TODO (#207) this is super freaking stupid inefficient.
         # IDEAS: batch it, don't get the queue until requested
@@ -263,9 +253,7 @@ class PlayerControls(Gtk.ActionBar):
             cover_art_result = AdapterManager.get_cover_art_uri(cover_art_id, "file")
             if not cover_art_result.data_is_available:
                 cover_art_result.add_done_callback(
-                    make_idle_index_capturing_function(
-                        idx, order_token, on_cover_art_future_done
-                    )
+                    make_idle_index_capturing_function(idx, order_token, on_cover_art_future_done)
                 )
                 return None
 
@@ -352,7 +340,7 @@ class PlayerControls(Gtk.ActionBar):
         cover_art_filename: str,
         app_config: AppConfiguration,
         force: bool = False,
-        order_token: int = None,
+        order_token: int | None = None,
         is_partial: bool = False,
     ):
         if order_token != self.cover_art_update_order_token:
@@ -361,6 +349,17 @@ class PlayerControls(Gtk.ActionBar):
         self.album_art.set_from_file(cover_art_filename)
         self.album_art.set_loading(False)
 
+    def update_rating(self, rating: int | None):
+        self.rating_buttons_box.rating = rating
+
+    def on_rating_clicked(self, _, rating: int):
+        if AdapterManager.can_set_song_rating():
+            self.emit("song-rated")
+
+    def on_rating_removed(self, _):
+        if AdapterManager.can_set_song_rating():
+            self.emit("song-rated")
+
     def update_scrubber(
         self,
         current: Optional[timedelta],
@@ -426,21 +425,15 @@ class PlayerControls(Gtk.ActionBar):
             return
 
         self._current_player_id = app_config.state.current_device
-        self._current_available_players = copy.deepcopy(
-            app_config.state.available_players
-        )
+        self._current_available_players = copy.deepcopy(app_config.state.available_players)
         for c in self.device_list.get_children():
             self.device_list.remove(c)
 
-        for i, (player_type, players) in enumerate(
-            app_config.state.available_players.items()
-        ):
+        for i, (player_type, players) in enumerate(app_config.state.available_players.items()):
             if len(players) == 0:
                 continue
             if i > 0:
-                self.device_list.add(
-                    Gtk.Separator(orientation=Gtk.Orientation.HORIZONTAL)
-                )
+                self.device_list.add(Gtk.Separator(orientation=Gtk.Orientation.HORIZONTAL))
             self.device_list.add(
                 Gtk.Label(
                     label=f"{player_type.name} Devices",
@@ -450,11 +443,7 @@ class PlayerControls(Gtk.ActionBar):
             )
 
             for player_id, player_name in sorted(players, key=lambda p: p[1]):
-                icon = (
-                    "audio-volume-high-symbolic"
-                    if player_id == self.current_device
-                    else None
-                )
+                icon = "audio-volume-high-symbolic" if player_id == self.current_device else None
                 button = IconButton(icon, label=player_name)
                 button.get_style_context().add_class("menu-button")
                 button.connect(
@@ -483,7 +472,7 @@ class PlayerControls(Gtk.ActionBar):
             def on_download_state_change(song_id: str):
                 # Refresh the entire window (no force) because the song could
                 # be in a list anywhere in the window.
-                self.emit("refresh-window", {}, False)
+                GLib.idle_add(lambda: self.emit("refresh-window", {}, False))
 
             # Use the new selection instead of the old one for calculating what
             # to do the right click on.
@@ -493,9 +482,7 @@ class PlayerControls(Gtk.ActionBar):
 
             song_ids = [self.play_queue_store[p][-1] for p in paths]
 
-            remove_text = (
-                "Remove " + util.pluralize("song", len(song_ids)) + " from queue"
-            )
+            remove_text = "Remove " + util.pluralize("song", len(song_ids)) + " from queue"
 
             def on_remove_songs_click(_: Any):
                 self.emit("songs-removed", [p.get_indices()[0] for p in paths])
@@ -592,9 +579,7 @@ class PlayerControls(Gtk.ActionBar):
         self.song_scrubber.set_name("song-scrubber")
         self.song_scrubber.set_draw_value(False)
         self.song_scrubber.set_restrict_to_fill_level(False)
-        self.song_scrubber.connect(
-            "change-value", lambda s, t, v: self.emit("song-scrub", v)
-        )
+        self.song_scrubber.connect("change-value", lambda s, t, v: self.emit("song-scrub", v))
         scrubber_box.pack_start(self.song_scrubber, True, True, 0)
 
         self.song_duration_label = Gtk.Label(label="-:--")
@@ -661,7 +646,14 @@ class PlayerControls(Gtk.ActionBar):
 
         return box
 
-    def create_play_queue_volume(self) -> Gtk.Box:
+    def create_rating_buttons(self):
+        # Rating button
+        self.rating_buttons_box = RatingButtonBox()
+        self.rating_buttons_box.set_property("sensitive", True)
+        self.rating_buttons_box.connect("rating-clicked", self.on_rating_clicked)
+        self.rating_buttons_box.connect("rating-remove", self.on_rating_removed)
+
+    def create_rating_play_queue_volume(self) -> Gtk.Box:
         vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         vbox.pack_start(Gtk.Box(), True, True, 0)
         box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
@@ -801,9 +793,7 @@ class PlayerControls(Gtk.ActionBar):
         self.play_queue_list.append_column(column)
 
         self.play_queue_list.connect("row-activated", self.on_song_activated)
-        self.play_queue_list.connect(
-            "button-press-event", self.on_play_queue_button_press
-        )
+        self.play_queue_list.connect("button-press-event", self.on_play_queue_button_press)
 
         # Set up drag-and-drop on the song list for editing the order of the
         # playlist.
@@ -825,9 +815,7 @@ class PlayerControls(Gtk.ActionBar):
         self.play_queue_popover.add(play_queue_popover_box)
 
         # Volume mute toggle
-        self.volume_mute_toggle = IconButton(
-            "audio-volume-high-symbolic", "Toggle mute"
-        )
+        self.volume_mute_toggle = IconButton("audio-volume-high-symbolic", "Toggle mute")
         self.volume_mute_toggle.set_action_name("app.mute-toggle")
         box.pack_start(self.volume_mute_toggle, False, True, 0)
 
@@ -840,6 +828,8 @@ class PlayerControls(Gtk.ActionBar):
         self.volume_slider.connect("value-changed", self.on_volume_change)
         box.pack_start(self.volume_slider, True, True, 0)
 
+        if AdapterManager.can_get_song_rating():
+            vbox.pack_start(self.rating_buttons_box, False, True, 0)
         vbox.pack_start(box, False, True, 0)
         vbox.pack_start(Gtk.Box(), True, True, 0)
         return vbox
