@@ -45,9 +45,7 @@ class UIState:
     @dataclass(unsafe_hash=True)
     class UINotification:
         markup: str
-        actions: Tuple[Tuple[str, Callable[[], None]], ...] = field(
-            default_factory=tuple
-        )
+        actions: Tuple[Tuple[str, Callable[[], None]], ...] = field(default_factory=tuple)
         icon: Optional[str] = None
 
     version: int = 1
@@ -87,10 +85,12 @@ class UIState:
         def __init__(self):
             self.name = "Rock"
 
-    current_album_search_query: AlbumSearchQuery = AlbumSearchQuery(
-        AlbumSearchQuery.Type.RANDOM,
-        genre=_DefaultGenre(),
-        year_range=this_decade(),
+    current_album_search_query: AlbumSearchQuery = field(
+        default_factory=lambda: AlbumSearchQuery(
+            AlbumSearchQuery.Type.RANDOM,
+            genre=UIState._DefaultGenre(),
+            year_range=this_decade(),
+        )
     )
 
     active_playlist_id: Optional[str] = None
@@ -112,9 +112,7 @@ class UIState:
     def __init_available_players__(self):
         from sublime_music.players import PlayerManager
 
-        self.available_players = {
-            pt: set() for pt in PlayerManager.available_player_types
-        }
+        self.available_players = {pt: set() for pt in PlayerManager.available_player_types}
 
     def migrate(self):
         pass
@@ -131,12 +129,31 @@ class UIState:
         current_song_id = self.play_queue[self.current_song_index]
 
         if not self._current_song or self._current_song.id != current_song_id:
-            self._current_song = AdapterManager.get_song_details(
-                current_song_id
-            ).result()
+            self._current_song = AdapterManager.get_song_details(current_song_id).result()
 
         return self._current_song
 
+    @property
+    def next_song_index(self) -> Optional[int]:
+        # If nothing is playing there is no next song
+        if self.current_song_index < 0:
+            return None
+
+        if self.repeat_type == RepeatType.REPEAT_SONG:
+            return self.current_song_index
+
+        # If we are at the end of the play queue
+        if self.current_song_index == len(self.play_queue) - 1:
+            # If we are repeating the queue, jump back to the beginning
+            if self.repeat_type == RepeatType.REPEAT_QUEUE:
+                return 0
+
+            # Otherwise, there isn't a next song
+            return None
+
+        # In all other cases, it's the song after the current one
+        return self.current_song_index + 1
+
     @property
     def volume(self) -> float:
         return self._volume.get(self.current_device, 100.0)
